import java.awt.*;

public class DigitalLineSegment {
	// 4-connected line in the 1st quadrant (0 and 1 directions)
	// mu <= ax - by < mu + a+ b 

	private int _a, _b, _mu; /* characteristics */
	private Point _U; /*upper leaning point of minimum abscissa*/
	private Point _L; /*lower leaning point of minimum abscissa*/
	private Point _Us;/*upper leaning point of maximum abscissa*/
	private Point _Ls;/*lower leaning point of maximum abscissa*/


	DigitalLineSegment(int firstMove){
		_U = new Point();
		_L = new Point();

		_mu = 0;

		if (firstMove == 0){
			_a = 0;
			_b = 1;
			_Us = new Point(1, 0);
			_Ls = new Point(1, 0);
		}
		else{
			_a = 1;
			_b = 0;
			_Us = new Point(0, 1);
			_Ls = new Point(0, 1);
		}
	}

	DigitalLineSegment(DigitalLineSegment d){
		_a = d._a;
		_b = d._b;
		_mu = d._mu;
		_U = new Point(d._U);
		_L = new Point(d._L);
		_Us = new Point(d._Us);
		_Ls = new Point(d._Ls);
	}

	DigitalLineSegment(){
		_a = 0;
		_b = 0;
		_mu = 0;
		_U = new Point();
		_L = new Point();
		_Us = new Point();
		_Ls = new Point();
	}


	public void copy(DigitalLineSegment d){
		_a = d._a;
		_b = d._b;
		_mu = d._mu;
		_U.setLocation(d._U);
		_L.setLocation(d._L);
		_Us.setLocation(d._Us);
		_Ls.setLocation(d._Ls);
	}

	public int getA(){
		return _a;
	}
	public int getB(){
		return _b;
	}

	// test if the point x y can be added to the line segment (without modifying the slope)
	boolean addTest (int x, int y){
		int r;
		r = _a*x-_b*y;
		return (_mu-1<=r && r<_mu+_b);
	}

	// Adds a point at the extremity of positive abscissa of a 4-connected line segment
	// in the 1st quadrant - adapted from Debled algorithm
	
	boolean addPositivePoint (int x, int y){
		int r;
		r = _a*x-_b*y;
		if(_mu<=r && r<_mu+_a+_b){
			if (r==_mu)
				_Us.move(x, y);
			if(r==_mu+_a+_b-1)
				_Ls.move(x, y);
			return true;
		}

		if(r==_mu-1){ // the slope increases
			//update of the leaning points
			_L.setLocation(_Ls);
			_Us.move(x, y);

			//update of the characteristics
			_a = y - (int)_U.getY();
			_b = x - (int)_U.getX();
			_mu = _a*x-_b*y;
			return true;
		}

		if(r==_mu+_a+_b){ // the slope decreases
			//update of the leaning points
			_U.setLocation(_Us);
			_Ls.move(x, y);

			//update of the characteristics
			_a = y - (int)_L.getY();
			_b = x - (int)_L.getX();
			_mu = _a*(x-1)-_b*(y+1)+1;
			return true;
		}
		// else the new point can not extend the segment
		return false;
	}

	// Adds a point at the extremity of negative abscissa of a 4-connected line segment
	// in the 1st quadrant - adapted from Debled algorithm
	
	boolean addNegativePoint(int x, int y){
		int r;
		r = _a*x-_b*y;
		if(_mu<=r && r< _mu+_a+_b){
			if (r==_mu)
				_U.move(x, y);
			if(r == _mu+_a+_b-1)
				_L.move(x, y);
			return true;
		}

		if(r==_mu-1){ //the slope decreases
			// update of the leaning points
			_Ls.setLocation(_L);
			_U.move(x, y);

			// update of the characteristics
			_a = (int)_Us.getY() - y;
			_b = (int)_Us.getX() - x;
			_mu = _a*x-_b*y;
			return true;
		}

		if(r==_mu+_a+_b){ // the slope increases
			// update of the leaning points
			_Us.setLocation(_U);
			_L.move(x, y);

			// update of the characteristics
			_a = (int)_Ls.getY()-y;
			_b = (int)_Ls.getX()-x;
			_mu = _a*(x-1)-_b*(y+1)+1;
			return true;
		}
		// else the new point can not extend the segment
		return false;
	}

}
