public class DigitalTangent {

	private DigitalLineSegment _d; // 4-c line in the 1st quadrant 
	private int _n; // symetric tangent of size 2n+1
	private int _dir0, _dir1; // even and odd directions composing the tangent, 
	// one of them can be not defined = -1
	private int _xm, _ym, _xM, _yM; // extreme points of the tangent in the 1st quadrant
	// axes x to the right and y upwards


	private boolean consistentMove (int dir) { // set _dir0 or _dir1 
		if (dir == _dir0 || dir == _dir1)
			return true;

		if (_dir0 == -1 && (dir == _dir1-1 || dir == (_dir1+1)%4)){
			_dir0 = dir;
			return true;
		}

		if (_dir1 == -1 && (dir == _dir0+1 || dir == (_dir0+3)%4)){
			_dir1 = dir;
			return true;
		}
		return false;
	}


	// symetric tangent computation by adding pairs of points
	DigitalTangent(FreemanCode c, int ind){
		int dir;
		int xM, yM, xm, ym;

		// initialization
		_n = 0;
		dir = c.getCode(ind, 1);

		if (dir%2 == 0){ // initial slope = 0
			xM = 1; 
			yM = 0;
			_dir0 = dir;
			_dir1 = -1;
			_d = new DigitalLineSegment(0);
		}
		else{ // initial slope = infinity
			xM = 0;
			yM = 1;
			_dir0 = -1;
			_dir1 = dir;
			_d = new DigitalLineSegment(1);
		}

		dir = c.getCode(ind, -1);
		consistentMove(dir); // always true
		DigitalLineSegment dSave = new DigitalLineSegment();

		if(dir%2 == 0){
			xm = -1; 
			ym = 0;
		}
		else{
			xm = 0; 
			ym = -1;
		}

		boolean ok = _d.addNegativePoint(xm, ym);
		int i = 2;
		while(ok){
			dSave.copy(_d);
			_xm = xm; _ym = ym;
			_xM = xM; _yM = yM;
			_n++;

			// Adds the positive point 
			dir = c.getCode(ind, i);
			if (consistentMove(dir)){
				xM += dir%2 == 0 ? 1 : 0;
				yM += dir%2 == 1 ? 1 : 0;
				ok = _d.addPositivePoint(xM, yM);
			}
			else
				ok = false;

			// Adds the neagtive point
			dir = c.getCode(ind, -i);
			if (ok && consistentMove(dir)){
				xm -= dir%2 ==0 ? 1 : 0;
				ym -= dir%2 ==1 ? 1 : 0;
				ok = _d.addNegativePoint(xm, ym);
			}
			else 
				ok = false;

			i++;
		}
		_d.copy(dSave);
	}


	public int getA(){
		return _d.getA();
	}

	public int getB(){
		return _d.getB();
	}

	public int getQuadrant(){
		if (_dir0 == -1)
			return _dir1;
		if (_dir1 == -1)
			return _dir0;
		if (_dir0 == 0 && _dir1 == 3)
			return 3;
		return _dir0 <= _dir1 ? _dir0 : _dir1;
	}

}
